@@ -354,8 +354,7 @@ try_spawn_config_tool (GdkScreen *screen)
                 gtk_window_set_title (GTK_WINDOW (no_tool_dialog), "");
                 gtk_window_set_resizable (GTK_WINDOW (no_tool_dialog), FALSE);
 
-                gtk_label_set_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (no_tool_dialog)->label),
-                                      str);
+                gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (no_tool_dialog), str);
 
                 g_free (str);
 
@@ -588,8 +587,7 @@ reload_mouse_modifiers (void)
         g_free (mouse_modifiers);
         mouse_modifiers = NULL;
 
-
-        n_mouse_modifiers = 2; /* control, alt */
+        n_mouse_modifiers = 1; /* alt */
         if (have_super)
                 ++n_mouse_modifiers;
         if (have_hyper)
@@ -597,18 +595,10 @@ reload_mouse_modifiers (void)
         if (have_meta)
                 ++n_mouse_modifiers;
 
-        g_free (mouse_modifiers);
-
         mouse_modifiers = g_new0 (MouseClickModifier, n_mouse_modifiers);
 
         i = 0;
 
-        mouse_modifiers[i].number = i;
-        /* translators: this is the Control key */
-        mouse_modifiers[i].name = g_strdup (_("C_ontrol"));
-        mouse_modifiers[i].value = "Control";
-        ++i;
-
         mouse_modifiers[i].number = i;
         mouse_modifiers[i].name = g_strdup (_("_Alt"));
         mouse_modifiers[i].value = "Alt";
